/*
ajax请求的主要js
 */
function imajax(){
	"use strict";
	/**
	 * 判断当前页面是否加载完成
	 * @type {Boolean}
	 */
	this.isComplete = true;
}

imajax.prototype = {

	run:function(){
		this.checkjs();
		this.linkClickAction();
		this.historyAction();
		this.commentAction();
	},

	checkjs:function(){
		$("html").removeClass("no-js");
	},


	/**
	 * 链接单击事件
	 */
	linkClickAction:function(){
		var _this = this;
		$("body").delegate("a",'click',function(){
			var href = $(this).attr("href");

			if ( !_this.isComplete ) {
				return false;
			}
			//不是外部链接,并且允许ajax
			if ( _this._allowAjax(href) ) {
				_this.afterCLickAnimate();
				_this.loadPage(href);
				return false;
			}

		});
	},

	/**
	 * 载入文章页面的动作
	 * @param {string} url 必须参数
	 * @param {string} referer 该url的来源，可以从页面的js获取，非必须参数
	 * @param {string} target 目标页面，非必须参数
	 * @return
	 */
	loadPage:function(url,referer,target){
		var _this = this;
		_this.isComplete = false;
		$.ajax({
			url:url,
			method:"GET",
			data:{"request":"ajax"},
			success:function(data){
				_this.processContent({
					"data":data,
					"referer":CURRENT_PAGE
				},function(args){
					args.href = url;
					_this.logHistory(args);
				});
				_this.isComplete = true;
			},
			error:function(e){
				//console.log(e);
			}
		});
	},

	/**
	 * 处理历史记录的问题
	 */
	historyAction:function(){

		if( !window.history.pushState ) return false;

		var state = {
			'title':document.title,
			'href':window.location.href,
			'content':$("#"+CURRENT_PAGE).html(),
			'referer':CURRENT_PAGE
		};
		this.logHistory(state);
		var _this = this;
		window.addEventListener("popstate",function(e){
			if( !e.state ) return false;
			//对内容进行处理
			_this.processContent(e.state,function(args){
				//callback
			});
		});
	},

	/**
	 * 记录历史
	 */
	logHistory:function(data){
		if( !window.history.pushState ) return false;

		state = {
			'title':data.title,
			'href':data.href,
			'content':data.content,
			'referer':CURRENT_PAGE
		};
		//console.log(state);
		window.history.pushState(state, data.title, data.href);
	},


	/**
	 * 处理内容
	 * @return 
	 */
	processContent:function(data,callback){
		if( data.data ){
			data.data = data.data.replace(/<!--.*?-->/g,'');
		}
		var dataDom = $(data.data)
		data.title = data.title || $(dataDom).find("#ajax-meta").attr("title") || document.title;
		data.content = data.content || $(dataDom).find('.right-content').html();
		data.target = data.target || $(dataDom).find("#ajax-meta").attr("data-page") || data.referer;
		data.href = data.href || window.location.href;

		//新的标题
		document.title = data.title;
		this.beforeNewDomLoadAnimate(data);
		$("#container").append('<div id="' + data.target + '" class="right-content">' + data.content + '</div>');
		//判断侧栏是否需要刷新
		var sidebar = dataDom.find("#sidebar").html();
		if(sidebar){
			if( sidebar.indexOf('document.write') != -1 ){
				sidebar = sidebar.replace(/<script[^>]*>.*<\/script>/,'');
			}
			$("#sidebar").html(sidebar);
		}
		this.afterNewDomLoadedAnimate(data);
		//更新页面类型
		CURRENT_PAGE = data.target;
		//刷新布局
		themeLayout.flushLayoutData();
		//回调函数
		if( callback ){

			callback({
				'title':data.title,
				'href': data.href,
				'content':data.content,
				'referer':data.target
			});

		}

	},

	/**
	 * 刷新当前页面
	 */
	flushContent:function(){
		this.loadPage(window.location.url,CURRENT_PAGE,CURRENT_PAGE);
	},

	/**
	 * 针对回复的动作
	 * 回复失败alert提示内容，回复成功则ajax刷新当前页面
	 */
	commentAction:function(){
		var _this = this;
		$("body").delegate("#commentform",'submit',function(){

			if( $(this).find("#comment").val() == '' ||
				$(this).find("#author").val() == '' ||
				$(this).find("#email").val() == '' 
			){
				alert("请填写必要字段");
				return false;
			}
			$(".cover").fadeIn(200);
			$.ajax({
				url:$(this).attr("action"),
				method:"POST",
				data:$(this).serialize(),
				success:function(data){
					if( data.indexOf('error-page') != -1 ){
						alert("评论失败,请检查评论字段");
						return false;
					}
					alert("评论成功");
					_this.flushContent();
				},
				error:function(e){
					switch(e.status){
						case 409:
							alert("这条评论似乎你已经评论过了");
						break;
						case 429:
							alert("你评论的速度太快啦，休息一下");
						break;
					}
				}
			});
			$(".cover").fadeOut(200);
			return false;
		});
	},


	/**
	 * 鼠标单击链接之后的动作
	 * @return
	 */
	afterCLickAnimate:function(){
		$("#container #"+CURRENT_PAGE).fadeOut(200);
		$(".cover").fadeIn(200);
	},

	/**
	 * 在新的页面载入之前所要执行的方法
	 * @return
	 */
	beforeNewDomLoadAnimate:function(data){
		this.rightContentTransform = $(".right-content").css("transform");
		$(".cover").fadeOut(200);
		$("#container #"+CURRENT_PAGE).remove();
	},

	/**
	 * 新的dom加载完成之后的动作
	 */
	afterNewDomLoadedAnimate:function(data){
		//恢复css
		$(".right-content").css({"transform":this.rightContentTransform});
		$("html,body").animate({scrollTop:"0px"}, 300);
		$("#"+data.target).hide().fadeIn(200);
		$(".cover").fadeOut(200);
		//重新绑定事件
		themeLayout.menuAction();
	},

	/**
	 * 判断是否是外部链接
	 * @param {string} href 当前链接
	 */
	_isExternal:function(href){
		if( !SITE_URL ) return true;
		if( href.indexOf(SITE_URL) === -1 )
			return true;
		return false;
	},

	/**
	 * 判断该链接是否允许ajax
	 */
	_allowAjax:function(href){
		//外部的不允许
		if (this._isExternal(href))
			return false;

		//对后台不允许ajax载入
		if (href.indexOf('wp-login') !== -1 ||
			href.indexOf('wp-admin') !== -1 ||
			href.indexOf('?replytocom') !== -1 ||
			href.indexOf('wp-content') !== -1
			) {
			return false;
		};
		return true;
	}


}

$(document).ready(function(){
	var ajax = new imajax();
	ajax.run();
});